--- **LibMouseGestures-1.0** Is a library for tracking and using mouse gestures
-- @class file
-- @name LibMouseGestures-1.0

--[[ LibMouseGestures-1.0

Revision: $Rev: 6 $
Author(s): Humbedooh
Description: Mouse gesture tracker
Dependencies: LibStub
License: MIT License

]]

--[===[@debug@
if ( dofile ) then
	dofile([[..\LibStub\LibStub.lua]]);
end
--@end-debug@]===]

local tinsert = table.insert;
local LIBMG = "LibMouseGestures-1.0"
local LIBMG_MINOR = tonumber(("$Rev: 06 $"):match("(%d+)")) or 10000;
if not LibStub then error(LIBMG .. " requires LibStub.") end
local LibMouseGestures = LibStub:NewLibrary(LIBMG, LIBMG_MINOR)
if not LibMouseGestures then return end

function LibMouseGestures:Setup(object)
	object.Update = LibMouseGestures.Update;
	object.UserInputReceived = LibMouseGestures.UserInputReceived;
	object.StartCapture = LibMouseGestures.StartCapture;
	object.GetBounds = LibMouseGestures.GetBounds;
	object.GetGist = LibMouseGestures.GetGist;
	object.GetMovementBounds = LibMouseGestures.GetMovementBounds;
	object.isRecording = false;
	object.mouseStart = {0,0};
	object.mouseEnd = {0,0};
	object.movements = {};
	return object;
end

function LibMouseGestures:New(frame)
	if ( type(frame) == "table" and frame.Show ) then
		local MG = LibMouseGestures:Setup({});
		MG.drawLayer = CreateFrame("Frame", frame);
		MG.drawLayer:SetParent(frame);
		MG.drawLayer:SetAllPoints(frame);
		MG.drawLayer:EnableMouse();
		MG.drawLayer:SetScript("OnMouseDown", function(caller,event) MG:UserInputReceived(event,"Down") end);
		MG.drawLayer:SetScript("OnMouseUp", function(caller,event) MG:UserInputReceived(event,"Up") end);
		MG.drawLayer:SetScript("OnUpdate", function(caller,event) MG:Update() end);
		MG.drawLayer:Hide(); -- Initially hidden
		return MG;
	else
		error(LIBMG .. ": Usage: LibMG:New(target_frame).");
	end
end

function LibMouseGestures:StartCapture(script)
	local MG = self;
	if ( type(script) ~= "table" ) then return; end
	MG.CaptureScript = script;
	MG.drawLayer:SetFrameLevel(9999);
	MG.drawLayer:Raise();
	MG.drawLayer:Show();
end

function LibMouseGestures:Update()
	local MG = self;
	local scale = UIParent:GetEffectiveScale();
	local cursorX, cursorY = GetCursorPosition();
	local script = MG.CaptureScript or {};
	MG.mouseEnd = { (cursorX - (MG.drawLayer:GetLeft()*scale))/scale, ((MG.drawLayer:GetTop()*scale) - cursorY)/scale };
	if ( MG.isRecording ) then
		if ( MG.drawLayer:IsMouseOver() ) then
			tinsert(MG.movements, { MG.mouseEnd[1], MG.mouseEnd[2] } );
			if ( type(script.updateFunc) == "function" ) then
				script.updateFunc(MG, MG.mouseStart[1], MG.mouseStart[2], MG.mouseEnd[1], MG.mouseEnd[2]);
			end
		end
	end
	if ( script.tooltip ) then
		GameTooltip:ClearLines();
		GameTooltip:SetOwner(MG.drawLayer, "ANCHOR_TOPLEFT", MG.mouseEnd[1]+32, -MG.mouseEnd[2]-48);
		if ( type(script.tooltip) == "string" ) then
			GameTooltip:SetText(script.tooltip);
		elseif (type(script.tooltip) == "table") then
			GameTooltip:SetText(script.tooltip[1]);
			for n = 2, #script.tooltip do
				GameTooltip:AddLine(script.tooltip[n]);
			end
		end
		GameTooltip:Show();
	end
end


function LibMouseGestures:UserInputReceived(button,action)
	local MG = self;
	local script = MG.CaptureScript or {};
	if ( type(script) ~= "table" ) then return; end
	if ( not MG.isRecording ) then
		if ( script.startButton == (button .. action) ) then
			MG.isRecording = true;
			MG.mouseStart = { MG.mouseEnd[1], MG.mouseEnd[2] };
			if ( type(script.startFunc) == "function" ) then
				script.startFunc(MG, MG.mouseStart[1], MG.mouseStart[2], MG.mouseEnd[1], MG.mouseEnd[2]);
			end
		elseif ( script.cancelButton == (button .. action) ) then
			MG.isRecording = false;
			if ( type(script.cancelFunc) == "function" ) then
				script.cancelFunc(MG, MG.mouseStart[1], MG.mouseStart[2], MG.mouseEnd[1], MG.mouseEnd[2]);
			end
			MG.CaptureScript = nil;
			GameTooltip:Hide();
			script = nil;
			MG.drawLayer:SetFrameLevel(0);
			MG.drawLayer:Hide();
		end
	else
		if ( script.stopButton == (button .. action) ) then
			MG.isRecording = false;
			if ( type(script.stopFunc) == "function" ) then
				script.stopFunc(MG, MG.mouseStart[1], MG.mouseStart[2], MG.mouseEnd[1], MG.mouseEnd[2]);
			end
			MG.CaptureScript = nil;
			MG.movements = {};
			GameTooltip:Hide();
			script = nil;
			MG.drawLayer:SetFrameLevel(0);
			MG.drawLayer:Hide();
		elseif ( script.cancelButton == (button .. action) ) then
			MG.isRecording = false;
			if ( type(script.cancelFunc) == "function" ) then
				script.cancelFunc(MG, MG.mouseStart[1], MG.mouseStart[2], MG.mouseEnd[1], MG.mouseEnd[2]);
			end
			MG.CaptureScript = nil;
			MG.movements = {};
			GameTooltip:Hide();
			script = nil;
			MG.drawLayer:SetFrameLevel(0);
			MG.drawLayer:Hide();
		elseif ( script.nextButton == (button .. action) ) then
			MG.mouseStart = { MG.mouseEnd[1], MG.mouseEnd[2]};
			if ( type(script.nextFunc) == "function" ) then
				script.nextFunc(MG, MG.mouseStart[1], MG.mouseStart[2], MG.mouseEnd[1], MG.mouseEnd[2]);
			end
			MG.movements = {};
		end
	end
end


function LibMouseGestures:GetBounds(shiftModifier)
	local MG = self;
	local sx,sy,ex,ey = MG.mouseStart[1], MG.mouseStart[2], MG.mouseEnd[1], MG.mouseEnd[2];
	local dx,dy = math.abs(sx-ex), math.abs(sy-ey);
	if ( dx < 5 ) then dx = 5; end
	if ( dy < 5 ) then dy = 5; end
	local a,b = math.min(sx,ex), math.min(sy,ey);
	if (shiftModifier and IsShiftKeyDown() ) then
		dx,dy = math.min(dx,dy), math.min(dx,dy);
	end
	return a,b,dx,dy;
end

function LibMouseGestures:GetMovementBounds()
	local MG = self;
	local mov = MG.movements;
	local a,b,c,d = nil,nil,nil,nil;
	for n = 1, #mov do
		local pos = mov[n];
		if ( a == nil ) then
			a,b,c,d = pos[1],pos[2],pos[1],pos[2];
		end
		if ( a > pos[1] ) then a = pos[1]; end
		if ( c < pos[1] ) then c = pos[1]; end
		if ( b > pos[2] ) then b = pos[2]; end
		if ( d < pos[2] ) then d = pos[2]; end
	end
	return a,b,c,d;
end


function LibMouseGestures:GetGist()
	local gists = {};
	local dirs = {
		{'right', 90},
		{'left', -90},
		{'up', 180},
		{'down',  0},
		{'rightup', 135},
		{'leftup', -135},
		{'rightdown', 45},
		{'leftdown', -45}
	}
	local md = 40; -- minimum travel distance before we consider a direction as being a gesture.
	local MG = self;
	local mov = MG.movements;
	local sx,sy,ex,ey = MG.mouseStart[1], MG.mouseStart[2], MG.mouseEnd[1], MG.mouseEnd[2];
	local a,b,c,d = MG:GetMovementBounds();
	local pangle = nil;
	if ( #mov > 1 ) then
		local ppos = mov[1];
		for n = 2, #mov do
			local cpos = mov[n];
			local dist = math.sqrt(math.pow(ppos[1] - cpos[1],2) + math.pow(ppos[2]-cpos[2], 2));
			if ( dist > md ) then
				local angle = math.floor( (math.deg(math.atan2(cpos[1] - ppos[1], cpos[2] - ppos[2])) / 45) + 0.5 ) * 45;
				if ( pangle == nil or ( math.abs(pangle-angle) > 37.5 ) ) then
					tdir = nil;
					for i = 1, #dirs do if ( angle == dirs[i][2] ) then tdir = dirs[i][1]; break; end end
					if ( tdir ) then tinsert(gists, {"line", tdir});
					--print("Moved in angle:", tdir);
					pangle = angle;
				end
				ppos = {cpos[1],cpos[2]};
			end
		end
	end
	return gists; -- ye ye, haven't finished this one yet.
end
